# this code has been automatically generated from pp/components/__init__.py
import pp


def test_grating_coupler_elliptical_te(data_regression):
    c = pp.c.grating_coupler_elliptical_te()
    data_regression.check(c.get_settings())


def test_rectangle(data_regression):
    c = pp.c.rectangle()
    data_regression.check(c.get_settings())


def test_hline(data_regression):
    c = pp.c.hline()
    data_regression.check(c.get_settings())


def test_bend_euler90(data_regression):
    c = pp.c.bend_euler90()
    data_regression.check(c.get_settings())


def test_ring(data_regression):
    c = pp.c.ring()
    data_regression.check(c.get_settings())


def test_spiral_inner_io_euler(data_regression):
    c = pp.c.spiral_inner_io_euler()
    data_regression.check(c.get_settings())


def test_crossing(data_regression):
    c = pp.c.crossing()
    data_regression.check(c.get_settings())


def test_grating_coupler_te(data_regression):
    c = pp.c.grating_coupler_te()
    data_regression.check(c.get_settings())


def test_litho_calipers(data_regression):
    c = pp.c.litho_calipers()
    data_regression.check(c.get_settings())


def test_mmi2x2(data_regression):
    c = pp.c.mmi2x2()
    data_regression.check(c.get_settings())


def test_test_resistance(data_regression):
    c = pp.c.test_resistance()
    data_regression.check(c.get_settings())


def test_spiral_inner_io(data_regression):
    c = pp.c.spiral_inner_io()
    data_regression.check(c.get_settings())


def test_compass(data_regression):
    c = pp.c.compass()
    data_regression.check(c.get_settings())


def test_coupler_symmetric(data_regression):
    c = pp.c.coupler_symmetric()
    data_regression.check(c.get_settings())


def test_grating_coupler_tm(data_regression):
    c = pp.c.grating_coupler_tm()
    data_regression.check(c.get_settings())


def test_spiral_external_io(data_regression):
    c = pp.c.spiral_external_io()
    data_regression.check(c.get_settings())


def test_coupler(data_regression):
    c = pp.c.coupler()
    data_regression.check(c.get_settings())


def test_spiral_circular(data_regression):
    c = pp.c.spiral_circular()
    data_regression.check(c.get_settings())


def test_grating_coupler_uniform(data_regression):
    c = pp.c.grating_coupler_uniform()
    data_regression.check(c.get_settings())


def test_text(data_regression):
    c = pp.c.text()
    data_regression.check(c.get_settings())


def test_wg_heater_connected(data_regression):
    c = pp.c.wg_heater_connected()
    data_regression.check(c.get_settings())


def test_bend_euler180(data_regression):
    c = pp.c.bend_euler180()
    data_regression.check(c.get_settings())


def test_test_via(data_regression):
    c = pp.c.test_via()
    data_regression.check(c.get_settings())


def test_litho_star(data_regression):
    c = pp.c.litho_star()
    data_regression.check(c.get_settings())


def test_pad(data_regression):
    c = pp.c.pad()
    data_regression.check(c.get_settings())


def test_loop_mirror(data_regression):
    c = pp.c.loop_mirror()
    data_regression.check(c.get_settings())


def test_ring_single_bus(data_regression):
    c = pp.c.ring_single_bus()
    data_regression.check(c.get_settings())


def test_component_lattice(data_regression):
    c = pp.c.component_lattice()
    data_regression.check(c.get_settings())


def test_bend_circular_heater(data_regression):
    c = pp.c.bend_circular_heater()
    data_regression.check(c.get_settings())


def test_delay_snake(data_regression):
    c = pp.c.delay_snake()
    data_regression.check(c.get_settings())


def test_grating_coupler_elliptical_tm(data_regression):
    c = pp.c.grating_coupler_elliptical_tm()
    data_regression.check(c.get_settings())


def test_waveguide_heater(data_regression):
    c = pp.c.waveguide_heater()
    data_regression.check(c.get_settings())


def test_bezier(data_regression):
    c = pp.c.bezier()
    data_regression.check(c.get_settings())


def test_cross(data_regression):
    c = pp.c.cross()
    data_regression.check(c.get_settings())


def test_circle(data_regression):
    c = pp.c.circle()
    data_regression.check(c.get_settings())


def test_pad_array(data_regression):
    c = pp.c.pad_array()
    data_regression.check(c.get_settings())


def test_bend_circular(data_regression):
    c = pp.c.bend_circular()
    data_regression.check(c.get_settings())


def test_rectangle_centered(data_regression):
    c = pp.c.rectangle_centered()
    data_regression.check(c.get_settings())


def test_coupler_asymmetric(data_regression):
    c = pp.c.coupler_asymmetric()
    data_regression.check(c.get_settings())


def test_bend_s(data_regression):
    c = pp.c.bend_s()
    data_regression.check(c.get_settings())


def test_taper(data_regression):
    c = pp.c.taper()
    data_regression.check(c.get_settings())


def test_grating_coupler_tree(data_regression):
    c = pp.c.grating_coupler_tree()
    data_regression.check(c.get_settings())


def test_mzi_arm(data_regression):
    c = pp.c.mzi_arm()
    data_regression.check(c.get_settings())


def test_bend_circular180(data_regression):
    c = pp.c.bend_circular180()
    data_regression.check(c.get_settings())


def test_taper_strip_to_ridge(data_regression):
    c = pp.c.taper_strip_to_ridge()
    data_regression.check(c.get_settings())


def test_waveguide(data_regression):
    c = pp.c.waveguide()
    data_regression.check(c.get_settings())


def test_litho_steps(data_regression):
    c = pp.c.litho_steps()
    data_regression.check(c.get_settings())


def test_coupler_straight(data_regression):
    c = pp.c.coupler_straight()
    data_regression.check(c.get_settings())


def test_mmi1x2(data_regression):
    c = pp.c.mmi1x2()
    data_regression.check(c.get_settings())


def test_ellipse(data_regression):
    c = pp.c.ellipse()
    data_regression.check(c.get_settings())


def test_coupler90(data_regression):
    c = pp.c.coupler90()
    data_regression.check(c.get_settings())


def test_ring_double_bus(data_regression):
    c = pp.c.ring_double_bus()
    data_regression.check(c.get_settings())
